package eu.mhsl.minenet.minigames.instance.game.stateless.types.tetris.game;

import eu.mhsl.minenet.minigames.instance.game.stateless.StatelessGame;
import eu.mhsl.minenet.minigames.util.BatchUtil;
import net.minestom.server.coordinate.Pos;
import net.minestom.server.entity.Entity;
import net.minestom.server.entity.EntityType;
import net.minestom.server.instance.batch.AbsoluteBlockBatch;
import net.minestom.server.instance.block.Block;

public class Playfield {
    private final Pos lowerLeftCorner;
    private final StatelessGame instance;
    private final static int height = 22;

    public Playfield(Pos lowerLeftCorner, StatelessGame instance) {
        this.lowerLeftCorner = lowerLeftCorner;
        this.instance = instance;
    }

    public Pos getPlayerSpawnPosition() {
//        return this.lowerLeftCorner.add(6.5, 9+((double) 3/16), 20.5).withView(180, 0);
        return this.lowerLeftCorner.add(6.5, 9, 20.5).withView(180, 0);
    }

    public Pos getTetrominoSpawnPosition() {
        return this.lowerLeftCorner.add(5, 21, 1);
    }

    public Pos getHoldPosition() {
        return this.lowerLeftCorner.add(-4, 18, 1);
    }

    public Pos getNextPosition() {
        return this.lowerLeftCorner.add(14, 18, 1);
    }

    public void generate() {
        AbsoluteBlockBatch batch = new AbsoluteBlockBatch();

        for(int x=0; x<12; x++) {
            for(int y = 0; y< height; y++) {
                batch.setBlock(this.lowerLeftCorner.add(x, y, 0), Block.GLASS);
                batch.setBlock(this.lowerLeftCorner.add(x, y, -1), Block.BLACK_CONCRETE);

                if(x==0 || x==11 || y==0) {
                    batch.setBlock(this.lowerLeftCorner.add(x, y, 1), Block.GRAY_CONCRETE);
                    batch.setBlock(this.lowerLeftCorner.add(x, y, 0), Block.GRAY_CONCRETE);
                }
            }
        }

        for(int x = 0; x < 6; x++) {
            for(int y = 0; y < 6; y++) {
                if(x==0 || x==5 || y==0 || y==5) {
                    batch.setBlock(this.getHoldPosition().add(x-2, y-2, 0), Block.BLACK_CONCRETE);
                    batch.setBlock(this.getNextPosition().add(x-2, y-2, 0), Block.BLACK_CONCRETE);
                    batch.setBlock(this.getHoldPosition().add(x-2, y-2, -1), Block.BLACK_CONCRETE);
                    batch.setBlock(this.getNextPosition().add(x-2, y-2, -1), Block.BLACK_CONCRETE);
                }
                batch.setBlock(this.getHoldPosition().add(x-2, y-2, -1), Block.QUARTZ_BLOCK);
                batch.setBlock(this.getNextPosition().add(x-2, y-2, -1), Block.QUARTZ_BLOCK);
            }
        }

        batch.setBlock(getPlayerSpawnPosition().sub(0, 1, 0), Block.STONE);

//        batch.setBlock(getPlayerSpawnPosition(), Block.IRON_TRAPDOOR.withProperty("half", "bottom"));
//        batch.setBlock(getPlayerSpawnPosition().add(0, 1, 0), Block.IRON_TRAPDOOR.withProperty("half", "top"));

        BatchUtil.loadAndApplyBatch(batch, this.instance, () -> {});
    }

    public int removeFullLines() {
        int removedLinesCounter = 0;
        for(int y = 1; y< height; y++) {
            boolean isFullLine = true;
            for(int x=1; x<11; x++) {
                if(this.instance.getBlock(this.lowerLeftCorner.add(x, y, 1)) == Block.AIR) isFullLine = false;
            }
            if(isFullLine) {
                removeFullLine(y);
                removedLinesCounter += 1;
                y -= 1;
            }
        }
        return removedLinesCounter;
    }

    public void removeEntity(EntityType entityType) {
        this.instance.getEntities().stream()
            .filter(entity -> entity.getEntityType().equals(entityType))
            .filter(entity -> {
                Pos position = entity.getPosition();
                if(position.x() > this.lowerLeftCorner.x() && position.y() > this.lowerLeftCorner.y()) {
                    return position.x() < this.lowerLeftCorner.x() + 11 && position.y() < this.lowerLeftCorner.y() + height;
                }
                return false;
            })
            .forEach(Entity::remove);
    }


    private void removeFullLine(int positionY) {
        for(int y = positionY; y< height; y++) {
            for(int x=1; x<11; x++) {
                Block blockAbove = this.instance.getBlock(this.lowerLeftCorner.add(x, y+1, 1));
                this.instance.setBlock(this.lowerLeftCorner.add(x, y, 1), blockAbove);
            }
        }
    }
}
